# -*- coding: utf-8 -*-
from typing import Union
from pathlib import Path

import ecdsa
from casperlabs_client.consts import SECP256K1_KEY_ALGORITHM
from casperlabs_client.io import read_binary_file
from .key_holder import KeyHolder


class SECP256K1Key(KeyHolder):
    """
    Class for loading, generating and handling public/private keys using secp256k1 algorithm

    Note: Many ecdsa methods are from/to_string. This is hold over from Python2 and work as bytes in Python3
    """

    CURVE = ecdsa.SECP256k1
    UNCOMPRESSED = "uncompressed"

    def __init__(
        self,
        private_key_pem: bytes = None,
        private_key=None,
        public_key_pem: bytes = None,
        public_key=None,
    ):
        super().__init__(
            private_key_pem,
            private_key,
            public_key_pem,
            public_key,
            SECP256K1_KEY_ALGORITHM,
        )

    def _private_key_pem_from_private_key(self) -> bytes:
        return ecdsa.SigningKey.from_string(self.private_key, curve=self.CURVE).to_pem()

    def _private_key_from_private_key_pem(self) -> bytes:
        return ecdsa.SigningKey.from_pem(
            self.private_key_pem.decode("UTF-8")
        ).to_string()

    def _public_key_pem_from_public_key(self) -> bytes:
        public_key = ecdsa.VerifyingKey.from_string(self.public_key, curve=self.CURVE)
        return public_key.to_pem()

    def _public_key_from_public_key_pem(self) -> bytes:
        return ecdsa.VerifyingKey.from_pem(
            self.public_key_pem.decode("UTF-8")
        ).to_string(self.UNCOMPRESSED)

    def _public_key_from_private_key(self) -> bytes:
        private_key = ecdsa.SigningKey.from_string(self.private_key, curve=self.CURVE)
        return private_key.verifying_key.to_string(self.UNCOMPRESSED)

    def sign(self, hashed_data: bytes) -> bytes:
        """ Return signature of data given """
        private_key = ecdsa.SigningKey.from_string(self.private_key, curve=self.CURVE)
        return private_key.sign_digest(
            hashed_data, sigencode=ecdsa.util.sigencode_der_canonize
        )

    @staticmethod
    def generate():
        """
        Generates a new key pair and returns as SEPC256K1Key object.

        :returns SECP256K1Key object
        """
        private_key_object = ecdsa.SigningKey.generate(curve=SECP256K1Key.CURVE)
        private_key = private_key_object.to_string()
        return SECP256K1Key(private_key=private_key)

    @staticmethod
    def from_private_key_path(private_key_pem_path: Union[str, Path]) -> "KeyHolder":
        """ Creates SECP256K1Key object from private key file in pem format """
        private_key_pem = read_binary_file(private_key_pem_path)
        return SECP256K1Key(private_key_pem=private_key_pem)

    @staticmethod
    def from_public_key_path(public_key_pem_path: Union[str, Path]) -> "KeyHolder":
        """
        Creates SECP256K1Key object from public key file in pem format.

        Note: Functionality requiring Private Key will not be possible.  Use only if no private key pem is available.
        """
        public_key_pem = read_binary_file(public_key_pem_path)
        return SECP256K1Key(public_key_pem=public_key_pem)

    @staticmethod
    def from_private_key(private_key: bytes) -> "KeyHolder":
        """ Creates SECP256K1Key object from private key in bytes """
        return SECP256K1Key(private_key=private_key)



# -*- coding: utf-8 -*-
import sys
from typing import Dict

from casperlabs_client import CasperLabsClient, io
from casperlabs_client.commands.common_options import ALGORITHM_OPTION
from casperlabs_client.decorators import guarded_command

NAME: str = "account-hash"
HELP: str = (
    "Calculate and return the account hash from public key and algorithm pair."
    "Saves in file if path given, otherwise uses stdout."
)
OPTIONS = (
    ALGORITHM_OPTION,
    (
        ("-k", "--public-key"),
        dict(required=True, type=str, help="Path to the file with account public key"),
    ),
    (
        ("-o", "--file-path"),
        dict(
            required=False,
            help=(
                "Path to the file where account hash will be saved. "
                "Optional, if not provided the account hash will be printed to STDOUT."
            ),
        ),
    ),
)


@guarded_command
def method(casperlabs_client: CasperLabsClient, args: Dict):
    algorithm = args.get("algorithm")
    public_key_path = args.get("public_key")
    file_path = args.get("file_path")

    account_hash = casperlabs_client.account_hash(
        algorithm=algorithm, public_key_pem_path=public_key_path
    )
    account_hash_hex = account_hash.hex().encode("UTF-8")

    if file_path:
        io.write_binary_file(file_path, account_hash_hex)
    else:
        sys.stdout.buffer.write(account_hash_hex)



# -*- coding: utf-8 -*-
import base64
from os.path import dirname, realpath
from typing import Union
from pathlib import Path

from casperlabs_client import consensus_pb2 as consensus
from casperlabs_client.reformat import hexify


def read_file(file_path: Union[Path, str]) -> str:
    with open(file_path, "r") as f:
        return f.read()


def read_binary_file(file_path: Union[Path, str]) -> bytes:
    with open(file_path, "rb") as f:
        return f.read()


def read_deploy_file(file_path: Union[Path, str]) -> consensus.Deploy:
    deploy = consensus.Deploy()
    file_contents = read_binary_file(file_path)
    deploy.ParseFromString(file_contents)
    return deploy


def write_file(file_path: Union[Path, str], text: str) -> None:
    with open(file_path, "w") as f:
        f.write(text)


def write_binary_file(file_path: Union[Path, str], data: bytes) -> None:
    with open(file_path, "wb") as f:
        f.write(data)


def read_pem_key(file_path: str) -> bytes:
    """ Read pem file and parse key

    :param file_path: Path to pem file
    """

    def is_key_line(line) -> bool:
        return line and not line.startswith("----")

    with open(file_path) as f:
        key_line = [line for line in f.readlines() if is_key_line(line)][0].strip()
        key_bytes = base64.b64decode(key_line)
        if len(key_bytes) % 32 == 0:
            return key_bytes[:32]
        else:
            return key_bytes[-32:]


def read_version() -> str:
    version_path = Path(dirname(realpath(__file__))) / "VERSION"
    return read_file(version_path).strip()


def print_blocks(response, element_name="block"):
    count = 0
    for block in response:
        print(f"------------- {element_name} {count} ---------------")
        print_block(block)
        print("-----------------------------------------------------\n")
        count += 1
    print("count:", count)


def print_block(block):
    print(hexify(block))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import diagnostics_pb2 as diagnostics__pb2
from . import empty_pb2 as empty__pb2


class DiagnosticsStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListPeers = channel.unary_unary(
                '/io.casperlabs.node.api.diagnostics.Diagnostics/ListPeers',
                request_serializer=empty__pb2.Empty.SerializeToString,
                response_deserializer=diagnostics__pb2.Peers.FromString,
                )


class DiagnosticsServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ListPeers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DiagnosticsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ListPeers': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPeers,
                    request_deserializer=empty__pb2.Empty.FromString,
                    response_serializer=diagnostics__pb2.Peers.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'io.casperlabs.node.api.diagnostics.Diagnostics', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Diagnostics(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ListPeers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.node.api.diagnostics.Diagnostics/ListPeers',
            empty__pb2.Empty.SerializeToString,
            diagnostics__pb2.Peers.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Hack to fix the relative imports problems with grpc #
import sys
from pathlib import Path

sys.path.append(str(Path(__file__).resolve().parents[1]))
# end of hack #

# ~/CasperLabs/protobuf/io/casperlabs/casper/consensus/info.proto
from . import info_pb2 as info


def deploy_info_view(full_view):
    if full_view:
        return info.DeployInfo.View.FULL
    else:
        return info.DeployInfo.View.BASIC


def block_info_view(full_view):
    if full_view:
        return info.BlockInfo.View.FULL
    else:
        return info.BlockInfo.View.BASIC



# -*- coding: utf-8 -*-
import sys
from typing import Dict

from casperlabs_client import CasperLabsClient
from casperlabs_client.decorators import guarded_command


NAME: str = "propose"
HELP: str = "[DEPRECATED] Force a node to propose a block based on its accumulated deploys."
OPTIONS = []


@guarded_command
def method(casperlabs_client: CasperLabsClient, args: Dict):
    print("Warning: method propose is deprecated.", file=sys.stderr)
    response = casperlabs_client.propose()
    print(f"Success! Block hash: {response.block_hash.hex()}")



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import ipc_pb2 as ipc__pb2


class ExecutionEngineServiceStub(object):
    """--- END PROOF-OF-STAKE SERVICE DEFINITION --- //

    Definition of the service.
    ExecutionEngine implements server part while Consensus implements client part.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.commit = channel.unary_unary(
                '/io.casperlabs.ipc.ExecutionEngineService/commit',
                request_serializer=ipc__pb2.CommitRequest.SerializeToString,
                response_deserializer=ipc__pb2.CommitResponse.FromString,
                )
        self.query = channel.unary_unary(
                '/io.casperlabs.ipc.ExecutionEngineService/query',
                request_serializer=ipc__pb2.QueryRequest.SerializeToString,
                response_deserializer=ipc__pb2.QueryResponse.FromString,
                )
        self.execute = channel.unary_unary(
                '/io.casperlabs.ipc.ExecutionEngineService/execute',
                request_serializer=ipc__pb2.ExecuteRequest.SerializeToString,
                response_deserializer=ipc__pb2.ExecuteResponse.FromString,
                )
        self.run_genesis = channel.unary_unary(
                '/io.casperlabs.ipc.ExecutionEngineService/run_genesis',
                request_serializer=ipc__pb2.RunGenesisRequest.SerializeToString,
                response_deserializer=ipc__pb2.GenesisResponse.FromString,
                )
        self.upgrade = channel.unary_unary(
                '/io.casperlabs.ipc.ExecutionEngineService/upgrade',
                request_serializer=ipc__pb2.UpgradeRequest.SerializeToString,
                response_deserializer=ipc__pb2.UpgradeResponse.FromString,
                )
        self.bid_state = channel.unary_unary(
                '/io.casperlabs.ipc.ExecutionEngineService/bid_state',
                request_serializer=ipc__pb2.BidStateRequest.SerializeToString,
                response_deserializer=ipc__pb2.BidStateResponse.FromString,
                )
        self.distribute_rewards = channel.unary_unary(
                '/io.casperlabs.ipc.ExecutionEngineService/distribute_rewards',
                request_serializer=ipc__pb2.DistributeRewardsRequest.SerializeToString,
                response_deserializer=ipc__pb2.DistributeRewardsResponse.FromString,
                )
        self.slash = channel.unary_unary(
                '/io.casperlabs.ipc.ExecutionEngineService/slash',
                request_serializer=ipc__pb2.SlashRequest.SerializeToString,
                response_deserializer=ipc__pb2.SlashResponse.FromString,
                )
        self.unbond_payout = channel.unary_unary(
                '/io.casperlabs.ipc.ExecutionEngineService/unbond_payout',
                request_serializer=ipc__pb2.UnbondPayoutRequest.SerializeToString,
                response_deserializer=ipc__pb2.UnbondPayoutResponse.FromString,
                )


class ExecutionEngineServiceServicer(object):
    """--- END PROOF-OF-STAKE SERVICE DEFINITION --- //

    Definition of the service.
    ExecutionEngine implements server part while Consensus implements client part.
    """

    def commit(self, request, context):
        """execution endpoints
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def query(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def execute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def run_genesis(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def upgrade(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def bid_state(self, request, context):
        """proof-of-stake endpoints
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def distribute_rewards(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def slash(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def unbond_payout(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ExecutionEngineServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'commit': grpc.unary_unary_rpc_method_handler(
                    servicer.commit,
                    request_deserializer=ipc__pb2.CommitRequest.FromString,
                    response_serializer=ipc__pb2.CommitResponse.SerializeToString,
            ),
            'query': grpc.unary_unary_rpc_method_handler(
                    servicer.query,
                    request_deserializer=ipc__pb2.QueryRequest.FromString,
                    response_serializer=ipc__pb2.QueryResponse.SerializeToString,
            ),
            'execute': grpc.unary_unary_rpc_method_handler(
                    servicer.execute,
                    request_deserializer=ipc__pb2.ExecuteRequest.FromString,
                    response_serializer=ipc__pb2.ExecuteResponse.SerializeToString,
            ),
            'run_genesis': grpc.unary_unary_rpc_method_handler(
                    servicer.run_genesis,
                    request_deserializer=ipc__pb2.RunGenesisRequest.FromString,
                    response_serializer=ipc__pb2.GenesisResponse.SerializeToString,
            ),
            'upgrade': grpc.unary_unary_rpc_method_handler(
                    servicer.upgrade,
                    request_deserializer=ipc__pb2.UpgradeRequest.FromString,
                    response_serializer=ipc__pb2.UpgradeResponse.SerializeToString,
            ),
            'bid_state': grpc.unary_unary_rpc_method_handler(
                    servicer.bid_state,
                    request_deserializer=ipc__pb2.BidStateRequest.FromString,
                    response_serializer=ipc__pb2.BidStateResponse.SerializeToString,
            ),
            'distribute_rewards': grpc.unary_unary_rpc_method_handler(
                    servicer.distribute_rewards,
                    request_deserializer=ipc__pb2.DistributeRewardsRequest.FromString,
                    response_serializer=ipc__pb2.DistributeRewardsResponse.SerializeToString,
            ),
            'slash': grpc.unary_unary_rpc_method_handler(
                    servicer.slash,
                    request_deserializer=ipc__pb2.SlashRequest.FromString,
                    response_serializer=ipc__pb2.SlashResponse.SerializeToString,
            ),
            'unbond_payout': grpc.unary_unary_rpc_method_handler(
                    servicer.unbond_payout,
                    request_deserializer=ipc__pb2.UnbondPayoutRequest.FromString,
                    response_serializer=ipc__pb2.UnbondPayoutResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'io.casperlabs.ipc.ExecutionEngineService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ExecutionEngineService(object):
    """--- END PROOF-OF-STAKE SERVICE DEFINITION --- //

    Definition of the service.
    ExecutionEngine implements server part while Consensus implements client part.
    """

    @staticmethod
    def commit(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.ipc.ExecutionEngineService/commit',
            ipc__pb2.CommitRequest.SerializeToString,
            ipc__pb2.CommitResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def query(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.ipc.ExecutionEngineService/query',
            ipc__pb2.QueryRequest.SerializeToString,
            ipc__pb2.QueryResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def execute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.ipc.ExecutionEngineService/execute',
            ipc__pb2.ExecuteRequest.SerializeToString,
            ipc__pb2.ExecuteResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def run_genesis(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.ipc.ExecutionEngineService/run_genesis',
            ipc__pb2.RunGenesisRequest.SerializeToString,
            ipc__pb2.GenesisResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def upgrade(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.ipc.ExecutionEngineService/upgrade',
            ipc__pb2.UpgradeRequest.SerializeToString,
            ipc__pb2.UpgradeResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def bid_state(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.ipc.ExecutionEngineService/bid_state',
            ipc__pb2.BidStateRequest.SerializeToString,
            ipc__pb2.BidStateResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def distribute_rewards(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.ipc.ExecutionEngineService/distribute_rewards',
            ipc__pb2.DistributeRewardsRequest.SerializeToString,
            ipc__pb2.DistributeRewardsResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def slash(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.ipc.ExecutionEngineService/slash',
            ipc__pb2.SlashRequest.SerializeToString,
            ipc__pb2.SlashResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def unbond_payout(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.ipc.ExecutionEngineService/unbond_payout',
            ipc__pb2.UnbondPayoutRequest.SerializeToString,
            ipc__pb2.UnbondPayoutResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
import abc
from dataclasses import dataclass
from typing import Dict, Union, Optional

from casperlabs_client.io import read_binary_file
from . import consensus_pb2 as consensus

from .abi import ABI


@dataclass
class ContractCode(abc.ABC):
    """ Abstract class representing many ways of sending in contracts to system """

    wasm_file_path: str = None
    contract_hash: bytes = None
    contract_name: str = None
    package_hash: bytes = None
    package_name: str = None
    contract_args: bytes = None
    entry_point: str = None
    version: int = None
    transfer_args: bytes = None

    def __post_init__(self):
        if self.contract_args is None:
            self.contract_args = ABI.args([])

    @staticmethod
    def _maybe_args_from_json(args: Union[str, bytes]) -> Optional[bytes]:
        """ Parses args from JSON format if exists """
        if args:
            # Encode args if still as str
            if isinstance(args, str):
                return ABI.args_from_json(args)
            else:
                return args
        return None

    @staticmethod
    @abc.abstractmethod
    def from_args(args: Dict) -> "ContractCode":
        pass

    @abc.abstractmethod
    def validate(self) -> None:
        """ Raises exception if object is built with invalid data """
        pass

    def is_valid(self) -> bool:
        """ Checks that object is built with valid data """
        try:
            self.validate()
            return True
        except Exception:
            return False

    def to_protobuf(self) -> consensus.Deploy.Code:
        """ Encode contract into consensus.Deploy.Code """
        if self.wasm_file_path:
            wasm_contract = consensus.Deploy.Code.WasmContract(
                wasm=read_binary_file(self.wasm_file_path)
            )
            return consensus.Deploy.Code(
                args=self.contract_args, wasm_contract=wasm_contract
            )
        if self.contract_hash:
            stored_contract = consensus.Deploy.Code.StoredContract(
                contract_hash=self.contract_hash, entry_point=self.entry_point
            )
            return consensus.Deploy.Code(
                args=self.contract_args, stored_contract=stored_contract
            )
        if self.contract_name:
            stored_contract = consensus.Deploy.Code.StoredContract(
                name=self.contract_name, entry_point=self.entry_point
            )
            return consensus.Deploy.Code(
                args=self.contract_args, stored_contract=stored_contract
            )
        elif self.package_hash:
            svc = consensus.Deploy.Code.StoredVersionedContract(
                package_hash=self.package_hash,
                entry_point=self.entry_point,
                version=self.version,
            )
            return consensus.Deploy.Code(
                args=self.contract_args, stored_versioned_contract=svc
            )
        elif self.package_name:
            svc = consensus.Deploy.Code.StoredVersionedContract(
                name=self.package_name,
                entry_point=self.entry_point,
                version=self.version,
            )
            return consensus.Deploy.Code(
                args=self.contract_args, stored_versioned_contract=svc
            )
        elif self.transfer_args:
            transfer_contract = consensus.Deploy.Code.TransferContract()
            return consensus.Deploy.Code(
                args=self.transfer_args, transfer_contract=transfer_contract
            )
        # If we fall through, may error or provide defaults
        return self._only_args_encode()

    @abc.abstractmethod
    def _only_args_encode(self) -> consensus.Deploy.Code:
        pass


@dataclass
class SessionCode(ContractCode):
    """ Representation of Session Code to send into the system. """

    def __post_init__(self):
        """ Use post init for validate to handle any creation method """
        self.validate()

    def validate(self) -> None:
        """ Validates the object and throws exceptions if problems are found"""
        session_options = (
            self.wasm_file_path,
            self.contract_name,
            self.contract_hash,
            self.package_hash,
            self.package_name,
            self.transfer_args,
        )

        options_count = len(list(filter(None, session_options)))
        if options_count != 1:
            raise ValueError(
                "Must have one and only one session, session_hash, session_name, "
                "session_package_hash, or session_package_name provided"
            )

    @staticmethod
    def from_args(args: Dict) -> "SessionCode":
        """ Creates SessionCode from CLI args """
        wasm_file_path = args.get("session")
        contract_hash = args.get("session_hash")
        contract_name = args.get("session_name")
        package_hash = args.get("session_package_hash")
        package_name = args.get("session_package_name")
        entry_point = args.get("session_entry_point")
        version = args.get("session_version")
        contract_args = SessionCode._maybe_args_from_json(args.get("session_args"))
        transfer_args = args.get("transfer_args")
        session_code = SessionCode(
            wasm_file_path=wasm_file_path,
            contract_args=contract_args,
            contract_hash=contract_hash,
            contract_name=contract_name,
            package_hash=package_hash,
            package_name=package_name,
            entry_point=entry_point,
            version=version,
            transfer_args=transfer_args,
        )
        return session_code

    def _only_args_encode(self) -> consensus.Deploy.Code:
        """ Should be caught in arg parsing for Deploy """
        raise ValueError("No runnable wasm or reference found for session.")


@dataclass
class PaymentCode(ContractCode):
    """ Representation of Payment Code to send into the system. """

    payment_amount: int = None

    @property
    def _payment_options(self) -> tuple:
        return (
            self.wasm_file_path,
            self.contract_name,
            self.contract_hash,
            self.package_name,
            self.package_hash,
            self.payment_amount,
        )

    def __post_init__(self):
        """ Use post init for validate to handle any creation method """
        if not any(self._payment_options):
            raise ValueError("No valid deploy payment options received.")
        self.validate()

    def validate(self) -> None:
        """ Validates the object and throws exceptions if problems are found"""
        options_count = len(list(filter(None, self._payment_options)))
        if options_count > 1:
            raise ValueError(
                "No more than one of payment, payment_hash, payment_name, payment_package_hash, "
                "payment_package_name, or payment_amount can be given."
            )
        elif options_count == 0:
            raise ValueError("No payment options were found.")

    @staticmethod
    def from_args(args: Dict) -> "PaymentCode":
        wasm_file_path = args.get("payment")
        payment_amount = args.get("payment_amount")
        contract_hash = args.get("payment_hash")
        contract_name = args.get("payment_name")
        package_hash = args.get("payment_package_hash")
        package_name = args.get("payment_package_name")
        entry_point = args.get("payment_entry_point")
        version = args.get("payment_version")
        contract_args = SessionCode._maybe_args_from_json(args.get("payment_args"))
        payment_code = PaymentCode(
            wasm_file_path=wasm_file_path,
            contract_args=contract_args,
            contract_hash=contract_hash,
            contract_name=contract_name,
            package_hash=package_hash,
            package_name=package_name,
            entry_point=entry_point,
            version=version,
            payment_amount=payment_amount,
        )
        return payment_code

    def _only_args_encode(self) -> consensus.Deploy.Code:
        """ Payment allows call with only args to standard payment """
        if self.contract_args:
            return consensus.Deploy.Code(args=self.contract_args)
        else:
            payment_args = ABI.args([ABI.big_int("amount", int(self.payment_amount))])
            return consensus.Deploy.Code(args=payment_args)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kademlia.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import node_pb2 as node__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='kademlia.proto',
  package='io.casperlabs.comm.discovery',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0ekademlia.proto\x12\x1cio.casperlabs.comm.discovery\x1a\nnode.proto\"A\n\x0bPingRequest\x12\x32\n\x06sender\x18\x01 \x01(\x0b\x32\".io.casperlabs.comm.discovery.Node\"\x0e\n\x0cPingResponse\"O\n\rLookupRequest\x12\n\n\x02id\x18\x01 \x01(\x0c\x12\x32\n\x06sender\x18\x02 \x01(\x0b\x32\".io.casperlabs.comm.discovery.Node\"C\n\x0eLookupResponse\x12\x31\n\x05nodes\x18\x01 \x03(\x0b\x32\".io.casperlabs.comm.discovery.Node2\xd9\x01\n\x0fKademliaService\x12_\n\x04Ping\x12).io.casperlabs.comm.discovery.PingRequest\x1a*.io.casperlabs.comm.discovery.PingResponse\"\x00\x12\x65\n\x06Lookup\x12+.io.casperlabs.comm.discovery.LookupRequest\x1a,.io.casperlabs.comm.discovery.LookupResponse\"\x00\x62\x06proto3'
  ,
  dependencies=[node__pb2.DESCRIPTOR,])




_PINGREQUEST = _descriptor.Descriptor(
  name='PingRequest',
  full_name='io.casperlabs.comm.discovery.PingRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender', full_name='io.casperlabs.comm.discovery.PingRequest.sender', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=125,
)


_PINGRESPONSE = _descriptor.Descriptor(
  name='PingResponse',
  full_name='io.casperlabs.comm.discovery.PingResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=127,
  serialized_end=141,
)


_LOOKUPREQUEST = _descriptor.Descriptor(
  name='LookupRequest',
  full_name='io.casperlabs.comm.discovery.LookupRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='io.casperlabs.comm.discovery.LookupRequest.id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender', full_name='io.casperlabs.comm.discovery.LookupRequest.sender', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=143,
  serialized_end=222,
)


_LOOKUPRESPONSE = _descriptor.Descriptor(
  name='LookupResponse',
  full_name='io.casperlabs.comm.discovery.LookupResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='nodes', full_name='io.casperlabs.comm.discovery.LookupResponse.nodes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=224,
  serialized_end=291,
)

_PINGREQUEST.fields_by_name['sender'].message_type = node__pb2._NODE
_LOOKUPREQUEST.fields_by_name['sender'].message_type = node__pb2._NODE
_LOOKUPRESPONSE.fields_by_name['nodes'].message_type = node__pb2._NODE
DESCRIPTOR.message_types_by_name['PingRequest'] = _PINGREQUEST
DESCRIPTOR.message_types_by_name['PingResponse'] = _PINGRESPONSE
DESCRIPTOR.message_types_by_name['LookupRequest'] = _LOOKUPREQUEST
DESCRIPTOR.message_types_by_name['LookupResponse'] = _LOOKUPRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PingRequest = _reflection.GeneratedProtocolMessageType('PingRequest', (_message.Message,), {
  'DESCRIPTOR' : _PINGREQUEST,
  '__module__' : 'kademlia_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.discovery.PingRequest)
  })
_sym_db.RegisterMessage(PingRequest)

PingResponse = _reflection.GeneratedProtocolMessageType('PingResponse', (_message.Message,), {
  'DESCRIPTOR' : _PINGRESPONSE,
  '__module__' : 'kademlia_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.discovery.PingResponse)
  })
_sym_db.RegisterMessage(PingResponse)

LookupRequest = _reflection.GeneratedProtocolMessageType('LookupRequest', (_message.Message,), {
  'DESCRIPTOR' : _LOOKUPREQUEST,
  '__module__' : 'kademlia_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.discovery.LookupRequest)
  })
_sym_db.RegisterMessage(LookupRequest)

LookupResponse = _reflection.GeneratedProtocolMessageType('LookupResponse', (_message.Message,), {
  'DESCRIPTOR' : _LOOKUPRESPONSE,
  '__module__' : 'kademlia_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.discovery.LookupResponse)
  })
_sym_db.RegisterMessage(LookupResponse)



_KADEMLIASERVICE = _descriptor.ServiceDescriptor(
  name='KademliaService',
  full_name='io.casperlabs.comm.discovery.KademliaService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=294,
  serialized_end=511,
  methods=[
  _descriptor.MethodDescriptor(
    name='Ping',
    full_name='io.casperlabs.comm.discovery.KademliaService.Ping',
    index=0,
    containing_service=None,
    input_type=_PINGREQUEST,
    output_type=_PINGRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Lookup',
    full_name='io.casperlabs.comm.discovery.KademliaService.Lookup',
    index=1,
    containing_service=None,
    input_type=_LOOKUPREQUEST,
    output_type=_LOOKUPRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_KADEMLIASERVICE)

DESCRIPTOR.services_by_name['KademliaService'] = _KADEMLIASERVICE

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: transforms.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import state_pb2 as state__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='transforms.proto',
  package='io.casperlabs.ipc',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x10transforms.proto\x12\x11io.casperlabs.ipc\x1a\x0bstate.proto\"/\n\x0cTypeMismatch\x12\x10\n\x08\x65xpected\x18\x01 \x01(\t\x12\r\n\x05\x66ound\x18\x02 \x01(\t\"\xb1\x03\n\tTransform\x12\x38\n\x08identity\x18\x01 \x01(\x0b\x32$.io.casperlabs.ipc.TransformIdentityH\x00\x12\x37\n\x07\x61\x64\x64_i32\x18\x02 \x01(\x0b\x32$.io.casperlabs.ipc.TransformAddInt32H\x00\x12\x38\n\x07\x61\x64\x64_u64\x18\x03 \x01(\x0b\x32%.io.casperlabs.ipc.TransformAddUInt64H\x00\x12\x32\n\x05write\x18\x04 \x01(\x0b\x32!.io.casperlabs.ipc.TransformWriteH\x00\x12\x37\n\x08\x61\x64\x64_keys\x18\x05 \x01(\x0b\x32#.io.casperlabs.ipc.TransformAddKeysH\x00\x12\x36\n\x07\x66\x61ilure\x18\x06 \x01(\x0b\x32#.io.casperlabs.ipc.TransformFailureH\x00\x12<\n\x0b\x61\x64\x64_big_int\x18\x07 \x01(\x0b\x32%.io.casperlabs.ipc.TransformAddBigIntH\x00\x42\x14\n\x12transform_instance\"\x13\n\x11TransformIdentity\"\"\n\x11TransformAddInt32\x12\r\n\x05value\x18\x01 \x01(\x05\"#\n\x12TransformAddUInt64\x12\r\n\x05value\x18\x01 \x01(\x04\"Q\n\x12TransformAddBigInt\x12;\n\x05value\x18\x01 \x01(\x0b\x32,.io.casperlabs.casper.consensus.state.BigInt\"Q\n\x10TransformAddKeys\x12=\n\x05value\x18\x01 \x03(\x0b\x32..io.casperlabs.casper.consensus.state.NamedKey\"R\n\x0eTransformWrite\x12@\n\x05value\x18\x01 \x01(\x0b\x32\x31.io.casperlabs.casper.consensus.state.StoredValue\"`\n\x10TransformFailure\x12\x38\n\rtype_mismatch\x18\x01 \x01(\x0b\x32\x1f.io.casperlabs.ipc.TypeMismatchH\x00\x42\x12\n\x10\x66\x61ilure_instance\"y\n\x0eTransformEntry\x12\x36\n\x03key\x18\x01 \x01(\x0b\x32).io.casperlabs.casper.consensus.state.Key\x12/\n\ttransform\x18\x02 \x01(\x0b\x32\x1c.io.casperlabs.ipc.Transformb\x06proto3'
  ,
  dependencies=[state__pb2.DESCRIPTOR,])




_TYPEMISMATCH = _descriptor.Descriptor(
  name='TypeMismatch',
  full_name='io.casperlabs.ipc.TypeMismatch',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='expected', full_name='io.casperlabs.ipc.TypeMismatch.expected', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='found', full_name='io.casperlabs.ipc.TypeMismatch.found', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=52,
  serialized_end=99,
)


_TRANSFORM = _descriptor.Descriptor(
  name='Transform',
  full_name='io.casperlabs.ipc.Transform',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='identity', full_name='io.casperlabs.ipc.Transform.identity', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='add_i32', full_name='io.casperlabs.ipc.Transform.add_i32', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='add_u64', full_name='io.casperlabs.ipc.Transform.add_u64', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='write', full_name='io.casperlabs.ipc.Transform.write', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='add_keys', full_name='io.casperlabs.ipc.Transform.add_keys', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='failure', full_name='io.casperlabs.ipc.Transform.failure', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='add_big_int', full_name='io.casperlabs.ipc.Transform.add_big_int', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='transform_instance', full_name='io.casperlabs.ipc.Transform.transform_instance',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=102,
  serialized_end=535,
)


_TRANSFORMIDENTITY = _descriptor.Descriptor(
  name='TransformIdentity',
  full_name='io.casperlabs.ipc.TransformIdentity',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=537,
  serialized_end=556,
)


_TRANSFORMADDINT32 = _descriptor.Descriptor(
  name='TransformAddInt32',
  full_name='io.casperlabs.ipc.TransformAddInt32',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='io.casperlabs.ipc.TransformAddInt32.value', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=558,
  serialized_end=592,
)


_TRANSFORMADDUINT64 = _descriptor.Descriptor(
  name='TransformAddUInt64',
  full_name='io.casperlabs.ipc.TransformAddUInt64',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='io.casperlabs.ipc.TransformAddUInt64.value', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=594,
  serialized_end=629,
)


_TRANSFORMADDBIGINT = _descriptor.Descriptor(
  name='TransformAddBigInt',
  full_name='io.casperlabs.ipc.TransformAddBigInt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='io.casperlabs.ipc.TransformAddBigInt.value', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=631,
  serialized_end=712,
)


_TRANSFORMADDKEYS = _descriptor.Descriptor(
  name='TransformAddKeys',
  full_name='io.casperlabs.ipc.TransformAddKeys',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='io.casperlabs.ipc.TransformAddKeys.value', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=714,
  serialized_end=795,
)


_TRANSFORMWRITE = _descriptor.Descriptor(
  name='TransformWrite',
  full_name='io.casperlabs.ipc.TransformWrite',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='io.casperlabs.ipc.TransformWrite.value', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=797,
  serialized_end=879,
)


_TRANSFORMFAILURE = _descriptor.Descriptor(
  name='TransformFailure',
  full_name='io.casperlabs.ipc.TransformFailure',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type_mismatch', full_name='io.casperlabs.ipc.TransformFailure.type_mismatch', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='failure_instance', full_name='io.casperlabs.ipc.TransformFailure.failure_instance',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=881,
  serialized_end=977,
)


_TRANSFORMENTRY = _descriptor.Descriptor(
  name='TransformEntry',
  full_name='io.casperlabs.ipc.TransformEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='io.casperlabs.ipc.TransformEntry.key', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transform', full_name='io.casperlabs.ipc.TransformEntry.transform', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=979,
  serialized_end=1100,
)

_TRANSFORM.fields_by_name['identity'].message_type = _TRANSFORMIDENTITY
_TRANSFORM.fields_by_name['add_i32'].message_type = _TRANSFORMADDINT32
_TRANSFORM.fields_by_name['add_u64'].message_type = _TRANSFORMADDUINT64
_TRANSFORM.fields_by_name['write'].message_type = _TRANSFORMWRITE
_TRANSFORM.fields_by_name['add_keys'].message_type = _TRANSFORMADDKEYS
_TRANSFORM.fields_by_name['failure'].message_type = _TRANSFORMFAILURE
_TRANSFORM.fields_by_name['add_big_int'].message_type = _TRANSFORMADDBIGINT
_TRANSFORM.oneofs_by_name['transform_instance'].fields.append(
  _TRANSFORM.fields_by_name['identity'])
_TRANSFORM.fields_by_name['identity'].containing_oneof = _TRANSFORM.oneofs_by_name['transform_instance']
_TRANSFORM.oneofs_by_name['transform_instance'].fields.append(
  _TRANSFORM.fields_by_name['add_i32'])
_TRANSFORM.fields_by_name['add_i32'].containing_oneof = _TRANSFORM.oneofs_by_name['transform_instance']
_TRANSFORM.oneofs_by_name['transform_instance'].fields.append(
  _TRANSFORM.fields_by_name['add_u64'])
_TRANSFORM.fields_by_name['add_u64'].containing_oneof = _TRANSFORM.oneofs_by_name['transform_instance']
_TRANSFORM.oneofs_by_name['transform_instance'].fields.append(
  _TRANSFORM.fields_by_name['write'])
_TRANSFORM.fields_by_name['write'].containing_oneof = _TRANSFORM.oneofs_by_name['transform_instance']
_TRANSFORM.oneofs_by_name['transform_instance'].fields.append(
  _TRANSFORM.fields_by_name['add_keys'])
_TRANSFORM.fields_by_name['add_keys'].containing_oneof = _TRANSFORM.oneofs_by_name['transform_instance']
_TRANSFORM.oneofs_by_name['transform_instance'].fields.append(
  _TRANSFORM.fields_by_name['failure'])
_TRANSFORM.fields_by_name['failure'].containing_oneof = _TRANSFORM.oneofs_by_name['transform_instance']
_TRANSFORM.oneofs_by_name['transform_instance'].fields.append(
  _TRANSFORM.fields_by_name['add_big_int'])
_TRANSFORM.fields_by_name['add_big_int'].containing_oneof = _TRANSFORM.oneofs_by_name['transform_instance']
_TRANSFORMADDBIGINT.fields_by_name['value'].message_type = state__pb2._BIGINT
_TRANSFORMADDKEYS.fields_by_name['value'].message_type = state__pb2._NAMEDKEY
_TRANSFORMWRITE.fields_by_name['value'].message_type = state__pb2._STOREDVALUE
_TRANSFORMFAILURE.fields_by_name['type_mismatch'].message_type = _TYPEMISMATCH
_TRANSFORMFAILURE.oneofs_by_name['failure_instance'].fields.append(
  _TRANSFORMFAILURE.fields_by_name['type_mismatch'])
_TRANSFORMFAILURE.fields_by_name['type_mismatch'].containing_oneof = _TRANSFORMFAILURE.oneofs_by_name['failure_instance']
_TRANSFORMENTRY.fields_by_name['key'].message_type = state__pb2._KEY
_TRANSFORMENTRY.fields_by_name['transform'].message_type = _TRANSFORM
DESCRIPTOR.message_types_by_name['TypeMismatch'] = _TYPEMISMATCH
DESCRIPTOR.message_types_by_name['Transform'] = _TRANSFORM
DESCRIPTOR.message_types_by_name['TransformIdentity'] = _TRANSFORMIDENTITY
DESCRIPTOR.message_types_by_name['TransformAddInt32'] = _TRANSFORMADDINT32
DESCRIPTOR.message_types_by_name['TransformAddUInt64'] = _TRANSFORMADDUINT64
DESCRIPTOR.message_types_by_name['TransformAddBigInt'] = _TRANSFORMADDBIGINT
DESCRIPTOR.message_types_by_name['TransformAddKeys'] = _TRANSFORMADDKEYS
DESCRIPTOR.message_types_by_name['TransformWrite'] = _TRANSFORMWRITE
DESCRIPTOR.message_types_by_name['TransformFailure'] = _TRANSFORMFAILURE
DESCRIPTOR.message_types_by_name['TransformEntry'] = _TRANSFORMENTRY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TypeMismatch = _reflection.GeneratedProtocolMessageType('TypeMismatch', (_message.Message,), {
  'DESCRIPTOR' : _TYPEMISMATCH,
  '__module__' : 'transforms_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.ipc.TypeMismatch)
  })
_sym_db.RegisterMessage(TypeMismatch)

Transform = _reflection.GeneratedProtocolMessageType('Transform', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORM,
  '__module__' : 'transforms_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.ipc.Transform)
  })
_sym_db.RegisterMessage(Transform)

TransformIdentity = _reflection.GeneratedProtocolMessageType('TransformIdentity', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMIDENTITY,
  '__module__' : 'transforms_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.ipc.TransformIdentity)
  })
_sym_db.RegisterMessage(TransformIdentity)

TransformAddInt32 = _reflection.GeneratedProtocolMessageType('TransformAddInt32', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMADDINT32,
  '__module__' : 'transforms_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.ipc.TransformAddInt32)
  })
_sym_db.RegisterMessage(TransformAddInt32)

TransformAddUInt64 = _reflection.GeneratedProtocolMessageType('TransformAddUInt64', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMADDUINT64,
  '__module__' : 'transforms_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.ipc.TransformAddUInt64)
  })
_sym_db.RegisterMessage(TransformAddUInt64)

TransformAddBigInt = _reflection.GeneratedProtocolMessageType('TransformAddBigInt', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMADDBIGINT,
  '__module__' : 'transforms_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.ipc.TransformAddBigInt)
  })
_sym_db.RegisterMessage(TransformAddBigInt)

TransformAddKeys = _reflection.GeneratedProtocolMessageType('TransformAddKeys', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMADDKEYS,
  '__module__' : 'transforms_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.ipc.TransformAddKeys)
  })
_sym_db.RegisterMessage(TransformAddKeys)

TransformWrite = _reflection.GeneratedProtocolMessageType('TransformWrite', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMWRITE,
  '__module__' : 'transforms_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.ipc.TransformWrite)
  })
_sym_db.RegisterMessage(TransformWrite)

TransformFailure = _reflection.GeneratedProtocolMessageType('TransformFailure', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMFAILURE,
  '__module__' : 'transforms_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.ipc.TransformFailure)
  })
_sym_db.RegisterMessage(TransformFailure)

TransformEntry = _reflection.GeneratedProtocolMessageType('TransformEntry', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMENTRY,
  '__module__' : 'transforms_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.ipc.TransformEntry)
  })
_sym_db.RegisterMessage(TransformEntry)


# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import consensus_pb2 as consensus__pb2
from . import empty_pb2 as empty__pb2
from . import gossiping_pb2 as gossiping__pb2


class GossipServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.NewBlocks = channel.unary_unary(
                '/io.casperlabs.comm.gossiping.GossipService/NewBlocks',
                request_serializer=gossiping__pb2.NewBlocksRequest.SerializeToString,
                response_deserializer=gossiping__pb2.NewBlocksResponse.FromString,
                )
        self.NewDeploys = channel.unary_unary(
                '/io.casperlabs.comm.gossiping.GossipService/NewDeploys',
                request_serializer=gossiping__pb2.NewDeploysRequest.SerializeToString,
                response_deserializer=gossiping__pb2.NewDeploysResponse.FromString,
                )
        self.StreamAncestorBlockSummaries = channel.unary_stream(
                '/io.casperlabs.comm.gossiping.GossipService/StreamAncestorBlockSummaries',
                request_serializer=gossiping__pb2.StreamAncestorBlockSummariesRequest.SerializeToString,
                response_deserializer=consensus__pb2.BlockSummary.FromString,
                )
        self.StreamLatestMessages = channel.unary_stream(
                '/io.casperlabs.comm.gossiping.GossipService/StreamLatestMessages',
                request_serializer=gossiping__pb2.StreamLatestMessagesRequest.SerializeToString,
                response_deserializer=consensus__pb2.Block.Justification.FromString,
                )
        self.StreamBlockSummaries = channel.unary_stream(
                '/io.casperlabs.comm.gossiping.GossipService/StreamBlockSummaries',
                request_serializer=gossiping__pb2.StreamBlockSummariesRequest.SerializeToString,
                response_deserializer=consensus__pb2.BlockSummary.FromString,
                )
        self.StreamDeploySummaries = channel.unary_stream(
                '/io.casperlabs.comm.gossiping.GossipService/StreamDeploySummaries',
                request_serializer=gossiping__pb2.StreamDeploySummariesRequest.SerializeToString,
                response_deserializer=consensus__pb2.DeploySummary.FromString,
                )
        self.GetBlockChunked = channel.unary_stream(
                '/io.casperlabs.comm.gossiping.GossipService/GetBlockChunked',
                request_serializer=gossiping__pb2.GetBlockChunkedRequest.SerializeToString,
                response_deserializer=gossiping__pb2.Chunk.FromString,
                )
        self.StreamDeploysChunked = channel.unary_stream(
                '/io.casperlabs.comm.gossiping.GossipService/StreamDeploysChunked',
                request_serializer=gossiping__pb2.StreamDeploysChunkedRequest.SerializeToString,
                response_deserializer=gossiping__pb2.Chunk.FromString,
                )
        self.GetGenesisCandidate = channel.unary_unary(
                '/io.casperlabs.comm.gossiping.GossipService/GetGenesisCandidate',
                request_serializer=gossiping__pb2.GetGenesisCandidateRequest.SerializeToString,
                response_deserializer=consensus__pb2.GenesisCandidate.FromString,
                )
        self.AddApproval = channel.unary_unary(
                '/io.casperlabs.comm.gossiping.GossipService/AddApproval',
                request_serializer=gossiping__pb2.AddApprovalRequest.SerializeToString,
                response_deserializer=empty__pb2.Empty.FromString,
                )
        self.StreamDagSliceBlockSummaries = channel.unary_stream(
                '/io.casperlabs.comm.gossiping.GossipService/StreamDagSliceBlockSummaries',
                request_serializer=gossiping__pb2.StreamDagSliceBlockSummariesRequest.SerializeToString,
                response_deserializer=consensus__pb2.BlockSummary.FromString,
                )


class GossipServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def NewBlocks(self, request, context):
        """Notify the callee about new blocks being available on the caller.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def NewDeploys(self, request, context):
        """Notify the callee about new deploys being available on the caller.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamAncestorBlockSummaries(self, request, context):
        """Retrieve the ancestors of certain blocks in the DAG; to be called repeatedly
        as necessary to synchronize DAGs between peers.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamLatestMessages(self, request, context):
        """Retrieve latest messages as the callee knows them.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamBlockSummaries(self, request, context):
        """Retrieve arbitrary block summaries, if the callee knows about them.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamDeploySummaries(self, request, context):
        """Retrieve arbitrary deploy summaries, if the callee knows about them.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBlockChunked(self, request, context):
        """Retrieve an arbitrarily sized block as a stream of chunks, with optionally compressed content.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamDeploysChunked(self, request, context):
        """Retrieve an arbitrary list of deploys, with optionally compressed contents.
        One use case would be to gossip deploys between nodes so that they can be included
        in a block by the first validator who gets to propose a block. The second case is
        to retrieve all the deploys which are missing from the body of a block, i.e. haven't
        been downloaded yet as part of earlier blocks or deploy gossiping.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetGenesisCandidate(self, request, context):
        """Retrieve the Genesis candidate supported by this node. Every node should either produce one from specification,
        get one from its bootstrap, or have it persisted from earlier runs.
        While the node is initializing and it hasn't obtained the candidate yet it will return UNAVAILABLE.
        Once the node is serving a candidate identity, it should also be prepared to serve the full block on request.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddApproval(self, request, context):
        """Add a signature to the list of approvals of a given candidate. If the block hash in the request doesn't match
        the callee's candidate it will return INVALID_ARGUMENT, otherwise add the signature and forward it to its peers.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamDagSliceBlockSummaries(self, request, context):
        """Returns block summaries between two ranks (inclusive) in ascending topological order.
        Used to gradually perform initial synchronization for new peers in the network.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_GossipServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'NewBlocks': grpc.unary_unary_rpc_method_handler(
                    servicer.NewBlocks,
                    request_deserializer=gossiping__pb2.NewBlocksRequest.FromString,
                    response_serializer=gossiping__pb2.NewBlocksResponse.SerializeToString,
            ),
            'NewDeploys': grpc.unary_unary_rpc_method_handler(
                    servicer.NewDeploys,
                    request_deserializer=gossiping__pb2.NewDeploysRequest.FromString,
                    response_serializer=gossiping__pb2.NewDeploysResponse.SerializeToString,
            ),
            'StreamAncestorBlockSummaries': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamAncestorBlockSummaries,
                    request_deserializer=gossiping__pb2.StreamAncestorBlockSummariesRequest.FromString,
                    response_serializer=consensus__pb2.BlockSummary.SerializeToString,
            ),
            'StreamLatestMessages': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamLatestMessages,
                    request_deserializer=gossiping__pb2.StreamLatestMessagesRequest.FromString,
                    response_serializer=consensus__pb2.Block.Justification.SerializeToString,
            ),
            'StreamBlockSummaries': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamBlockSummaries,
                    request_deserializer=gossiping__pb2.StreamBlockSummariesRequest.FromString,
                    response_serializer=consensus__pb2.BlockSummary.SerializeToString,
            ),
            'StreamDeploySummaries': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamDeploySummaries,
                    request_deserializer=gossiping__pb2.StreamDeploySummariesRequest.FromString,
                    response_serializer=consensus__pb2.DeploySummary.SerializeToString,
            ),
            'GetBlockChunked': grpc.unary_stream_rpc_method_handler(
                    servicer.GetBlockChunked,
                    request_deserializer=gossiping__pb2.GetBlockChunkedRequest.FromString,
                    response_serializer=gossiping__pb2.Chunk.SerializeToString,
            ),
            'StreamDeploysChunked': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamDeploysChunked,
                    request_deserializer=gossiping__pb2.StreamDeploysChunkedRequest.FromString,
                    response_serializer=gossiping__pb2.Chunk.SerializeToString,
            ),
            'GetGenesisCandidate': grpc.unary_unary_rpc_method_handler(
                    servicer.GetGenesisCandidate,
                    request_deserializer=gossiping__pb2.GetGenesisCandidateRequest.FromString,
                    response_serializer=consensus__pb2.GenesisCandidate.SerializeToString,
            ),
            'AddApproval': grpc.unary_unary_rpc_method_handler(
                    servicer.AddApproval,
                    request_deserializer=gossiping__pb2.AddApprovalRequest.FromString,
                    response_serializer=empty__pb2.Empty.SerializeToString,
            ),
            'StreamDagSliceBlockSummaries': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamDagSliceBlockSummaries,
                    request_deserializer=gossiping__pb2.StreamDagSliceBlockSummariesRequest.FromString,
                    response_serializer=consensus__pb2.BlockSummary.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'io.casperlabs.comm.gossiping.GossipService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class GossipService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def NewBlocks(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.comm.gossiping.GossipService/NewBlocks',
            gossiping__pb2.NewBlocksRequest.SerializeToString,
            gossiping__pb2.NewBlocksResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def NewDeploys(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.comm.gossiping.GossipService/NewDeploys',
            gossiping__pb2.NewDeploysRequest.SerializeToString,
            gossiping__pb2.NewDeploysResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamAncestorBlockSummaries(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/io.casperlabs.comm.gossiping.GossipService/StreamAncestorBlockSummaries',
            gossiping__pb2.StreamAncestorBlockSummariesRequest.SerializeToString,
            consensus__pb2.BlockSummary.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamLatestMessages(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/io.casperlabs.comm.gossiping.GossipService/StreamLatestMessages',
            gossiping__pb2.StreamLatestMessagesRequest.SerializeToString,
            consensus__pb2.Block.Justification.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamBlockSummaries(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/io.casperlabs.comm.gossiping.GossipService/StreamBlockSummaries',
            gossiping__pb2.StreamBlockSummariesRequest.SerializeToString,
            consensus__pb2.BlockSummary.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamDeploySummaries(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/io.casperlabs.comm.gossiping.GossipService/StreamDeploySummaries',
            gossiping__pb2.StreamDeploySummariesRequest.SerializeToString,
            consensus__pb2.DeploySummary.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetBlockChunked(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/io.casperlabs.comm.gossiping.GossipService/GetBlockChunked',
            gossiping__pb2.GetBlockChunkedRequest.SerializeToString,
            gossiping__pb2.Chunk.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamDeploysChunked(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/io.casperlabs.comm.gossiping.GossipService/StreamDeploysChunked',
            gossiping__pb2.StreamDeploysChunkedRequest.SerializeToString,
            gossiping__pb2.Chunk.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetGenesisCandidate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.comm.gossiping.GossipService/GetGenesisCandidate',
            gossiping__pb2.GetGenesisCandidateRequest.SerializeToString,
            consensus__pb2.GenesisCandidate.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddApproval(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.comm.gossiping.GossipService/AddApproval',
            gossiping__pb2.AddApprovalRequest.SerializeToString,
            empty__pb2.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamDagSliceBlockSummaries(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/io.casperlabs.comm.gossiping.GossipService/StreamDagSliceBlockSummaries',
            gossiping__pb2.StreamDagSliceBlockSummariesRequest.SerializeToString,
            consensus__pb2.BlockSummary.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import control_pb2 as control__pb2


class ControlServiceStub(object):
    """The operators can use the ControlService to issue commands to the node,
    such as proposing a block or otherwise influence the proposal strategy.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Propose = channel.unary_unary(
                '/io.casperlabs.node.api.control.ControlService/Propose',
                request_serializer=control__pb2.ProposeRequest.SerializeToString,
                response_deserializer=control__pb2.ProposeResponse.FromString,
                )


class ControlServiceServicer(object):
    """The operators can use the ControlService to issue commands to the node,
    such as proposing a block or otherwise influence the proposal strategy.
    """

    def Propose(self, request, context):
        """Propose a block using the deploys in the pool.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ControlServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Propose': grpc.unary_unary_rpc_method_handler(
                    servicer.Propose,
                    request_deserializer=control__pb2.ProposeRequest.FromString,
                    response_serializer=control__pb2.ProposeResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'io.casperlabs.node.api.control.ControlService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ControlService(object):
    """The operators can use the ControlService to issue commands to the node,
    such as proposing a block or otherwise influence the proposal strategy.
    """

    @staticmethod
    def Propose(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/io.casperlabs.node.api.control.ControlService/Propose',
            control__pb2.ProposeRequest.SerializeToString,
            control__pb2.ProposeResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: info.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import consensus_pb2 as consensus__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='info.proto',
  package='io.casperlabs.casper.consensus.info',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\ninfo.proto\x12#io.casperlabs.casper.consensus.info\x1a\x0f\x63onsensus.proto\"\xa9\x04\n\tBlockInfo\x12=\n\x07summary\x18\x01 \x01(\x0b\x32,.io.casperlabs.casper.consensus.BlockSummary\x12\x45\n\x06status\x18\x02 \x01(\x0b\x32\x35.io.casperlabs.casper.consensus.info.BlockInfo.Status\x1a\xf8\x02\n\x06Status\x12J\n\x05stats\x18\x02 \x01(\x0b\x32;.io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats\x12\x14\n\x0c\x63hild_hashes\x18\x03 \x03(\x0c\x12P\n\x08\x66inality\x18\x05 \x01(\x0e\x32>.io.casperlabs.casper.consensus.info.BlockInfo.Status.Finality\x1av\n\x05Stats\x12\x18\n\x10\x62lock_size_bytes\x18\x01 \x01(\r\x12\x1a\n\x12\x64\x65ploy_error_count\x18\x02 \x01(\r\x12\x19\n\x11\x64\x65ploy_cost_total\x18\x03 \x01(\x04\x12\x1c\n\x14\x64\x65ploy_gas_price_avg\x18\x04 \x01(\x04\"6\n\x08\x46inality\x12\r\n\tUNDECIDED\x10\x00\x12\r\n\tFINALIZED\x10\x01\x12\x0c\n\x08ORPHANED\x10\x02J\x04\x08\x01\x10\x02J\x04\x08\x04\x10\x05\"\x1b\n\x04View\x12\t\n\x05\x42\x41SIC\x10\x00\x12\x08\n\x04\x46ULL\x10\x01\"\xd9\x04\n\nDeployInfo\x12\x36\n\x06\x64\x65ploy\x18\x01 \x01(\x0b\x32&.io.casperlabs.casper.consensus.Deploy\x12\\\n\x12processing_results\x18\x02 \x03(\x0b\x32@.io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult\x12\x46\n\x06status\x18\x03 \x01(\x0b\x32\x36.io.casperlabs.casper.consensus.info.DeployInfo.Status\x1a\x9c\x01\n\x10ProcessingResult\x12\x42\n\nblock_info\x18\x01 \x01(\x0b\x32..io.casperlabs.casper.consensus.info.BlockInfo\x12\x0c\n\x04\x63ost\x18\x02 \x01(\x04\x12\x10\n\x08is_error\x18\x03 \x01(\x08\x12\x15\n\rerror_message\x18\x04 \x01(\t\x12\r\n\x05stage\x18\x05 \x01(\r\x1a_\n\x06Status\x12\x44\n\x05state\x18\x01 \x01(\x0e\x32\x35.io.casperlabs.casper.consensus.info.DeployInfo.State\x12\x0f\n\x07message\x18\x02 \x01(\t\"\x1b\n\x04View\x12\t\n\x05\x42\x41SIC\x10\x00\x12\x08\n\x04\x46ULL\x10\x01\"P\n\x05State\x12\r\n\tUNDEFINED\x10\x00\x12\x0b\n\x07PENDING\x10\x01\x12\r\n\tPROCESSED\x10\x02\x12\r\n\tFINALIZED\x10\x03\x12\r\n\tDISCARDED\x10\x04\"\xe5\x0b\n\x05\x45vent\x12\x10\n\x08\x65vent_id\x18\t \x01(\x04\x12L\n\x0b\x62lock_added\x18\x01 \x01(\x0b\x32\x35.io.casperlabs.casper.consensus.info.Event.BlockAddedH\x00\x12[\n\x13new_finalized_block\x18\x02 \x01(\x0b\x32<.io.casperlabs.casper.consensus.info.Event.NewFinalizedBlockH\x00\x12N\n\x0c\x64\x65ploy_added\x18\x03 \x01(\x0b\x32\x36.io.casperlabs.casper.consensus.info.Event.DeployAddedH\x00\x12T\n\x0f\x64\x65ploy_requeued\x18\x04 \x01(\x0b\x32\x39.io.casperlabs.casper.consensus.info.Event.DeployRequeuedH\x00\x12V\n\x10\x64\x65ploy_discarded\x18\x05 \x01(\x0b\x32:.io.casperlabs.casper.consensus.info.Event.DeployDiscardedH\x00\x12V\n\x10\x64\x65ploy_processed\x18\x06 \x01(\x0b\x32:.io.casperlabs.casper.consensus.info.Event.DeployProcessedH\x00\x12V\n\x10\x64\x65ploy_finalized\x18\x07 \x01(\x0b\x32:.io.casperlabs.casper.consensus.info.Event.DeployFinalizedH\x00\x12T\n\x0f\x64\x65ploy_orphaned\x18\x08 \x01(\x0b\x32\x39.io.casperlabs.casper.consensus.info.Event.DeployOrphanedH\x00\x1aK\n\nBlockAdded\x12=\n\x05\x62lock\x18\x01 \x01(\x0b\x32..io.casperlabs.casper.consensus.info.BlockInfo\x1a|\n\x11NewFinalizedBlock\x12\x12\n\nblock_hash\x18\x01 \x01(\x0c\x12)\n!indirectly_finalized_block_hashes\x18\x02 \x03(\x0c\x12(\n indirectly_orphaned_block_hashes\x18\x03 \x03(\x0c\x1a\x45\n\x0b\x44\x65ployAdded\x12\x36\n\x06\x64\x65ploy\x18\x01 \x01(\x0b\x32&.io.casperlabs.casper.consensus.Deploy\x1aZ\n\x0f\x44\x65ployDiscarded\x12\x36\n\x06\x64\x65ploy\x18\x01 \x01(\x0b\x32&.io.casperlabs.casper.consensus.Deploy\x12\x0f\n\x07message\x18\x02 \x01(\t\x1aH\n\x0e\x44\x65ployRequeued\x12\x36\n\x06\x64\x65ploy\x18\x01 \x01(\x0b\x32&.io.casperlabs.casper.consensus.Deploy\x1av\n\x0f\x44\x65ployProcessed\x12\x12\n\nblock_hash\x18\x01 \x01(\x0c\x12O\n\x10processed_deploy\x18\x02 \x01(\x0b\x32\x35.io.casperlabs.casper.consensus.Block.ProcessedDeploy\x1av\n\x0f\x44\x65ployFinalized\x12\x12\n\nblock_hash\x18\x01 \x01(\x0c\x12O\n\x10processed_deploy\x18\x02 \x01(\x0b\x32\x35.io.casperlabs.casper.consensus.Block.ProcessedDeploy\x1aj\n\x0e\x44\x65ployOrphaned\x12\x12\n\nblock_hash\x18\x01 \x01(\x0c\x12\x44\n\x0b\x64\x65ploy_info\x18\x02 \x01(\x0b\x32/.io.casperlabs.casper.consensus.info.DeployInfoB\x07\n\x05valueb\x06proto3'
  ,
  dependencies=[consensus__pb2.DESCRIPTOR,])



_BLOCKINFO_STATUS_FINALITY = _descriptor.EnumDescriptor(
  name='Finality',
  full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.Finality',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNDECIDED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FINALIZED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ORPHANED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=527,
  serialized_end=581,
)
_sym_db.RegisterEnumDescriptor(_BLOCKINFO_STATUS_FINALITY)

_BLOCKINFO_VIEW = _descriptor.EnumDescriptor(
  name='View',
  full_name='io.casperlabs.casper.consensus.info.BlockInfo.View',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BASIC', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FULL', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=595,
  serialized_end=622,
)
_sym_db.RegisterEnumDescriptor(_BLOCKINFO_VIEW)

_DEPLOYINFO_VIEW = _descriptor.EnumDescriptor(
  name='View',
  full_name='io.casperlabs.casper.consensus.info.DeployInfo.View',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BASIC', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FULL', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=595,
  serialized_end=622,
)
_sym_db.RegisterEnumDescriptor(_DEPLOYINFO_VIEW)

_DEPLOYINFO_STATE = _descriptor.EnumDescriptor(
  name='State',
  full_name='io.casperlabs.casper.consensus.info.DeployInfo.State',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNDEFINED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PENDING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PROCESSED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FINALIZED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DISCARDED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1146,
  serialized_end=1226,
)
_sym_db.RegisterEnumDescriptor(_DEPLOYINFO_STATE)


_BLOCKINFO_STATUS_STATS = _descriptor.Descriptor(
  name='Stats',
  full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_size_bytes', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats.block_size_bytes', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deploy_error_count', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats.deploy_error_count', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deploy_cost_total', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats.deploy_cost_total', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deploy_gas_price_avg', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats.deploy_gas_price_avg', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=407,
  serialized_end=525,
)

_BLOCKINFO_STATUS = _descriptor.Descriptor(
  name='Status',
  full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='stats', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.stats', index=0,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='child_hashes', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.child_hashes', index=1,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='finality', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.finality', index=2,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_BLOCKINFO_STATUS_STATS, ],
  enum_types=[
    _BLOCKINFO_STATUS_FINALITY,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=217,
  serialized_end=593,
)

_BLOCKINFO = _descriptor.Descriptor(
  name='BlockInfo',
  full_name='io.casperlabs.casper.consensus.info.BlockInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='summary', full_name='io.casperlabs.casper.consensus.info.BlockInfo.summary', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='io.casperlabs.casper.consensus.info.BlockInfo.status', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_BLOCKINFO_STATUS, ],
  enum_types=[
    _BLOCKINFO_VIEW,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=69,
  serialized_end=622,
)


_DEPLOYINFO_PROCESSINGRESULT = _descriptor.Descriptor(
  name='ProcessingResult',
  full_name='io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_info', full_name='io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult.block_info', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cost', full_name='io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult.cost', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_error', full_name='io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult.is_error', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error_message', full_name='io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult.error_message', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stage', full_name='io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult.stage', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=862,
  serialized_end=1018,
)

_DEPLOYINFO_STATUS = _descriptor.Descriptor(
  name='Status',
  full_name='io.casperlabs.casper.consensus.info.DeployInfo.Status',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='io.casperlabs.casper.consensus.info.DeployInfo.Status.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='io.casperlabs.casper.consensus.info.DeployInfo.Status.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1020,
  serialized_end=1115,
)

_DEPLOYINFO = _descriptor.Descriptor(
  name='DeployInfo',
  full_name='io.casperlabs.casper.consensus.info.DeployInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deploy', full_name='io.casperlabs.casper.consensus.info.DeployInfo.deploy', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='processing_results', full_name='io.casperlabs.casper.consensus.info.DeployInfo.processing_results', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='io.casperlabs.casper.consensus.info.DeployInfo.status', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DEPLOYINFO_PROCESSINGRESULT, _DEPLOYINFO_STATUS, ],
  enum_types=[
    _DEPLOYINFO_VIEW,
    _DEPLOYINFO_STATE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=625,
  serialized_end=1226,
)


_EVENT_BLOCKADDED = _descriptor.Descriptor(
  name='BlockAdded',
  full_name='io.casperlabs.casper.consensus.info.Event.BlockAdded',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='block', full_name='io.casperlabs.casper.consensus.info.Event.BlockAdded.block', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1943,
  serialized_end=2018,
)

_EVENT_NEWFINALIZEDBLOCK = _descriptor.Descriptor(
  name='NewFinalizedBlock',
  full_name='io.casperlabs.casper.consensus.info.Event.NewFinalizedBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_hash', full_name='io.casperlabs.casper.consensus.info.Event.NewFinalizedBlock.block_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='indirectly_finalized_block_hashes', full_name='io.casperlabs.casper.consensus.info.Event.NewFinalizedBlock.indirectly_finalized_block_hashes', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='indirectly_orphaned_block_hashes', full_name='io.casperlabs.casper.consensus.info.Event.NewFinalizedBlock.indirectly_orphaned_block_hashes', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2020,
  serialized_end=2144,
)

_EVENT_DEPLOYADDED = _descriptor.Descriptor(
  name='DeployAdded',
  full_name='io.casperlabs.casper.consensus.info.Event.DeployAdded',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deploy', full_name='io.casperlabs.casper.consensus.info.Event.DeployAdded.deploy', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2146,
  serialized_end=2215,
)

_EVENT_DEPLOYDISCARDED = _descriptor.Descriptor(
  name='DeployDiscarded',
  full_name='io.casperlabs.casper.consensus.info.Event.DeployDiscarded',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deploy', full_name='io.casperlabs.casper.consensus.info.Event.DeployDiscarded.deploy', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='io.casperlabs.casper.consensus.info.Event.DeployDiscarded.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2217,
  serialized_end=2307,
)

_EVENT_DEPLOYREQUEUED = _descriptor.Descriptor(
  name='DeployRequeued',
  full_name='io.casperlabs.casper.consensus.info.Event.DeployRequeued',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deploy', full_name='io.casperlabs.casper.consensus.info.Event.DeployRequeued.deploy', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2309,
  serialized_end=2381,
)

_EVENT_DEPLOYPROCESSED = _descriptor.Descriptor(
  name='DeployProcessed',
  full_name='io.casperlabs.casper.consensus.info.Event.DeployProcessed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_hash', full_name='io.casperlabs.casper.consensus.info.Event.DeployProcessed.block_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='processed_deploy', full_name='io.casperlabs.casper.consensus.info.Event.DeployProcessed.processed_deploy', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2383,
  serialized_end=2501,
)

_EVENT_DEPLOYFINALIZED = _descriptor.Descriptor(
  name='DeployFinalized',
  full_name='io.casperlabs.casper.consensus.info.Event.DeployFinalized',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_hash', full_name='io.casperlabs.casper.consensus.info.Event.DeployFinalized.block_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='processed_deploy', full_name='io.casperlabs.casper.consensus.info.Event.DeployFinalized.processed_deploy', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2503,
  serialized_end=2621,
)

_EVENT_DEPLOYORPHANED = _descriptor.Descriptor(
  name='DeployOrphaned',
  full_name='io.casperlabs.casper.consensus.info.Event.DeployOrphaned',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_hash', full_name='io.casperlabs.casper.consensus.info.Event.DeployOrphaned.block_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deploy_info', full_name='io.casperlabs.casper.consensus.info.Event.DeployOrphaned.deploy_info', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2623,
  serialized_end=2729,
)

_EVENT = _descriptor.Descriptor(
  name='Event',
  full_name='io.casperlabs.casper.consensus.info.Event',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event_id', full_name='io.casperlabs.casper.consensus.info.Event.event_id', index=0,
      number=9, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='block_added', full_name='io.casperlabs.casper.consensus.info.Event.block_added', index=1,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='new_finalized_block', full_name='io.casperlabs.casper.consensus.info.Event.new_finalized_block', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deploy_added', full_name='io.casperlabs.casper.consensus.info.Event.deploy_added', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deploy_requeued', full_name='io.casperlabs.casper.consensus.info.Event.deploy_requeued', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deploy_discarded', full_name='io.casperlabs.casper.consensus.info.Event.deploy_discarded', index=5,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deploy_processed', full_name='io.casperlabs.casper.consensus.info.Event.deploy_processed', index=6,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deploy_finalized', full_name='io.casperlabs.casper.consensus.info.Event.deploy_finalized', index=7,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deploy_orphaned', full_name='io.casperlabs.casper.consensus.info.Event.deploy_orphaned', index=8,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_EVENT_BLOCKADDED, _EVENT_NEWFINALIZEDBLOCK, _EVENT_DEPLOYADDED, _EVENT_DEPLOYDISCARDED, _EVENT_DEPLOYREQUEUED, _EVENT_DEPLOYPROCESSED, _EVENT_DEPLOYFINALIZED, _EVENT_DEPLOYORPHANED, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='io.casperlabs.casper.consensus.info.Event.value',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1229,
  serialized_end=2738,
)

_BLOCKINFO_STATUS_STATS.containing_type = _BLOCKINFO_STATUS
_BLOCKINFO_STATUS.fields_by_name['stats'].message_type = _BLOCKINFO_STATUS_STATS
_BLOCKINFO_STATUS.fields_by_name['finality'].enum_type = _BLOCKINFO_STATUS_FINALITY
_BLOCKINFO_STATUS.containing_type = _BLOCKINFO
_BLOCKINFO_STATUS_FINALITY.containing_type = _BLOCKINFO_STATUS
_BLOCKINFO.fields_by_name['summary'].message_type = consensus__pb2._BLOCKSUMMARY
_BLOCKINFO.fields_by_name['status'].message_type = _BLOCKINFO_STATUS
_BLOCKINFO_VIEW.containing_type = _BLOCKINFO
_DEPLOYINFO_PROCESSINGRESULT.fields_by_name['block_info'].message_type = _BLOCKINFO
_DEPLOYINFO_PROCESSINGRESULT.containing_type = _DEPLOYINFO
_DEPLOYINFO_STATUS.fields_by_name['state'].enum_type = _DEPLOYINFO_STATE
_DEPLOYINFO_STATUS.containing_type = _DEPLOYINFO
_DEPLOYINFO.fields_by_name['deploy'].message_type = consensus__pb2._DEPLOY
_DEPLOYINFO.fields_by_name['processing_results'].message_type = _DEPLOYINFO_PROCESSINGRESULT
_DEPLOYINFO.fields_by_name['status'].message_type = _DEPLOYINFO_STATUS
_DEPLOYINFO_VIEW.containing_type = _DEPLOYINFO
_DEPLOYINFO_STATE.containing_type = _DEPLOYINFO
_EVENT_BLOCKADDED.fields_by_name['block'].message_type = _BLOCKINFO
_EVENT_BLOCKADDED.containing_type = _EVENT
_EVENT_NEWFINALIZEDBLOCK.containing_type = _EVENT
_EVENT_DEPLOYADDED.fields_by_name['deploy'].message_type = consensus__pb2._DEPLOY
_EVENT_DEPLOYADDED.containing_type = _EVENT
_EVENT_DEPLOYDISCARDED.fields_by_name['deploy'].message_type = consensus__pb2._DEPLOY
_EVENT_DEPLOYDISCARDED.containing_type = _EVENT
_EVENT_DEPLOYREQUEUED.fields_by_name['deploy'].message_type = consensus__pb2._DEPLOY
_EVENT_DEPLOYREQUEUED.containing_type = _EVENT
_EVENT_DEPLOYPROCESSED.fields_by_name['processed_deploy'].message_type = consensus__pb2._BLOCK_PROCESSEDDEPLOY
_EVENT_DEPLOYPROCESSED.containing_type = _EVENT
_EVENT_DEPLOYFINALIZED.fields_by_name['processed_deploy'].message_type = consensus__pb2._BLOCK_PROCESSEDDEPLOY
_EVENT_DEPLOYFINALIZED.containing_type = _EVENT
_EVENT_DEPLOYORPHANED.fields_by_name['deploy_info'].message_type = _DEPLOYINFO
_EVENT_DEPLOYORPHANED.containing_type = _EVENT
_EVENT.fields_by_name['block_added'].message_type = _EVENT_BLOCKADDED
_EVENT.fields_by_name['new_finalized_block'].message_type = _EVENT_NEWFINALIZEDBLOCK
_EVENT.fields_by_name['deploy_added'].message_type = _EVENT_DEPLOYADDED
_EVENT.fields_by_name['deploy_requeued'].message_type = _EVENT_DEPLOYREQUEUED
_EVENT.fields_by_name['deploy_discarded'].message_type = _EVENT_DEPLOYDISCARDED
_EVENT.fields_by_name['deploy_processed'].message_type = _EVENT_DEPLOYPROCESSED
_EVENT.fields_by_name['deploy_finalized'].message_type = _EVENT_DEPLOYFINALIZED
_EVENT.fields_by_name['deploy_orphaned'].message_type = _EVENT_DEPLOYORPHANED
_EVENT.oneofs_by_name['value'].fields.append(
  _EVENT.fields_by_name['block_added'])
_EVENT.fields_by_name['block_added'].containing_oneof = _EVENT.oneofs_by_name['value']
_EVENT.oneofs_by_name['value'].fields.append(
  _EVENT.fields_by_name['new_finalized_block'])
_EVENT.fields_by_name['new_finalized_block'].containing_oneof = _EVENT.oneofs_by_name['value']
_EVENT.oneofs_by_name['value'].fields.append(
  _EVENT.fields_by_name['deploy_added'])
_EVENT.fields_by_name['deploy_added'].containing_oneof = _EVENT.oneofs_by_name['value']
_EVENT.oneofs_by_name['value'].fields.append(
  _EVENT.fields_by_name['deploy_requeued'])
_EVENT.fields_by_name['deploy_requeued'].containing_oneof = _EVENT.oneofs_by_name['value']
_EVENT.oneofs_by_name['value'].fields.append(
  _EVENT.fields_by_name['deploy_discarded'])
_EVENT.fields_by_name['deploy_discarded'].containing_oneof = _EVENT.oneofs_by_name['value']
_EVENT.oneofs_by_name['value'].fields.append(
  _EVENT.fields_by_name['deploy_processed'])
_EVENT.fields_by_name['deploy_processed'].containing_oneof = _EVENT.oneofs_by_name['value']
_EVENT.oneofs_by_name['value'].fields.append(
  _EVENT.fields_by_name['deploy_finalized'])
_EVENT.fields_by_name['deploy_finalized'].containing_oneof = _EVENT.oneofs_by_name['value']
_EVENT.oneofs_by_name['value'].fields.append(
  _EVENT.fields_by_name['deploy_orphaned'])
_EVENT.fields_by_name['deploy_orphaned'].containing_oneof = _EVENT.oneofs_by_name['value']
DESCRIPTOR.message_types_by_name['BlockInfo'] = _BLOCKINFO
DESCRIPTOR.message_types_by_name['DeployInfo'] = _DEPLOYINFO
DESCRIPTOR.message_types_by_name['Event'] = _EVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BlockInfo = _reflection.GeneratedProtocolMessageType('BlockInfo', (_message.Message,), {

  'Status' : _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), {

    'Stats' : _reflection.GeneratedProtocolMessageType('Stats', (_message.Message,), {
      'DESCRIPTOR' : _BLOCKINFO_STATUS_STATS,
      '__module__' : 'info_pb2'
      # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats)
      })
    ,
    'DESCRIPTOR' : _BLOCKINFO_STATUS,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.BlockInfo.Status)
    })
  ,
  'DESCRIPTOR' : _BLOCKINFO,
  '__module__' : 'info_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.BlockInfo)
  })
_sym_db.RegisterMessage(BlockInfo)
_sym_db.RegisterMessage(BlockInfo.Status)
_sym_db.RegisterMessage(BlockInfo.Status.Stats)

DeployInfo = _reflection.GeneratedProtocolMessageType('DeployInfo', (_message.Message,), {

  'ProcessingResult' : _reflection.GeneratedProtocolMessageType('ProcessingResult', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYINFO_PROCESSINGRESULT,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult)
    })
  ,

  'Status' : _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYINFO_STATUS,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.DeployInfo.Status)
    })
  ,
  'DESCRIPTOR' : _DEPLOYINFO,
  '__module__' : 'info_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.DeployInfo)
  })
_sym_db.RegisterMessage(DeployInfo)
_sym_db.RegisterMessage(DeployInfo.ProcessingResult)
_sym_db.RegisterMessage(DeployInfo.Status)

Event = _reflection.GeneratedProtocolMessageType('Event', (_message.Message,), {

  'BlockAdded' : _reflection.GeneratedProtocolMessageType('BlockAdded', (_message.Message,), {
    'DESCRIPTOR' : _EVENT_BLOCKADDED,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.Event.BlockAdded)
    })
  ,

  'NewFinalizedBlock' : _reflection.GeneratedProtocolMessageType('NewFinalizedBlock', (_message.Message,), {
    'DESCRIPTOR' : _EVENT_NEWFINALIZEDBLOCK,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.Event.NewFinalizedBlock)
    })
  ,

  'DeployAdded' : _reflection.GeneratedProtocolMessageType('DeployAdded', (_message.Message,), {
    'DESCRIPTOR' : _EVENT_DEPLOYADDED,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.Event.DeployAdded)
    })
  ,

  'DeployDiscarded' : _reflection.GeneratedProtocolMessageType('DeployDiscarded', (_message.Message,), {
    'DESCRIPTOR' : _EVENT_DEPLOYDISCARDED,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.Event.DeployDiscarded)
    })
  ,

  'DeployRequeued' : _reflection.GeneratedProtocolMessageType('DeployRequeued', (_message.Message,), {
    'DESCRIPTOR' : _EVENT_DEPLOYREQUEUED,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.Event.DeployRequeued)
    })
  ,

  'DeployProcessed' : _reflection.GeneratedProtocolMessageType('DeployProcessed', (_message.Message,), {
    'DESCRIPTOR' : _EVENT_DEPLOYPROCESSED,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.Event.DeployProcessed)
    })
  ,

  'DeployFinalized' : _reflection.GeneratedProtocolMessageType('DeployFinalized', (_message.Message,), {
    'DESCRIPTOR' : _EVENT_DEPLOYFINALIZED,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.Event.DeployFinalized)
    })
  ,

  'DeployOrphaned' : _reflection.GeneratedProtocolMessageType('DeployOrphaned', (_message.Message,), {
    'DESCRIPTOR' : _EVENT_DEPLOYORPHANED,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.Event.DeployOrphaned)
    })
  ,
  'DESCRIPTOR' : _EVENT,
  '__module__' : 'info_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.Event)
  })
_sym_db.RegisterMessage(Event)
_sym_db.RegisterMessage(Event.BlockAdded)
_sym_db.RegisterMessage(Event.NewFinalizedBlock)
_sym_db.RegisterMessage(Event.DeployAdded)
_sym_db.RegisterMessage(Event.DeployDiscarded)
_sym_db.RegisterMessage(Event.DeployRequeued)
_sym_db.RegisterMessage(Event.DeployProcessed)
_sym_db.RegisterMessage(Event.DeployFinalized)
_sym_db.RegisterMessage(Event.DeployOrphaned)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
from casperlabs_client import CasperLabsClient, consts, reformat
from casperlabs_client.commands.common_options import (
    FROM_OPTION,
    CHAINNAME_OPTION,
    DEPENDENCIES_OPTION,
    TTL_MILLIS_OPTION,
    private_key_option,
    WAIT_PROCESSED_OPTION,
    TIMEOUT_SECONDS_OPTION,
    ALGORITHM_OPTION,
    PAYMENT_OPTIONS,
)
from casperlabs_client.decorators import guarded_command


NAME: str = "transfer"
HELP: str = "Transfers funds between accounts"
OPTIONS = [
    [
        ("-a", "--amount"),
        dict(
            required=True,
            default=None,
            type=int,
            help="Amount of motes to transfer. Note: a mote is the smallest, indivisible unit of a token.",
        ),
    ],
    [
        ("-t", "--target-account"),
        dict(
            required=False,
            type=str,
            help="base64 or base16 representation of target account's public key",
        ),
    ],
    [
        ("--target-purse",),
        dict(
            required=False,
            type=str,
            help="base64 or base16 representation of target purse URef",
        ),
    ],
    [
        ("--source-purse",),
        dict(
            required=False,
            type=str,
            help="base64 or base16 representation of source purse URef",
        ),
    ],
    FROM_OPTION,
    CHAINNAME_OPTION,
    DEPENDENCIES_OPTION,
    TTL_MILLIS_OPTION,
    WAIT_PROCESSED_OPTION,
    TIMEOUT_SECONDS_OPTION,
    ALGORITHM_OPTION,
    private_key_option(required=True),
] + PAYMENT_OPTIONS


@guarded_command
def method(casperlabs_client: CasperLabsClient, args: dict):
    deploy_hash = casperlabs_client.transfer(
        amount=args.get("amount"),
        target_account=args.get("target_account"),
        target_purse=args.get("target_purse"),
        source_purse=args.get("source_purse"),
        from_addr=args.get("from_addr"),
        private_key=args.get("private_key"),
        ttl_millis=args.get("ttl_millis"),
        dependencies=args.get("dependencies"),
        chain_name=args.get("chain_name"),
        algorithm=args.get("algorithm"),
        payment=args.get("payment"),
        payment_amount=args.get("payment_amount"),
        payment_args=args.get("payment_args"),
        payment_hash=args.get("payment_hash"),
        payment_name=args.get("payment_name"),
        payment_entry_point=args.get("payment_entry_point"),
        payment_package_hash=args.get("payment_package_hash"),
        payment_package_name=args.get("payment_package_name"),
    )
    print(f"Success! Deploy {deploy_hash} deployed")
    if args.get("wait_for_processed", False):
        deploy_info = casperlabs_client.showDeploy(
            deploy_hash,
            full_view=False,
            wait_for_processed=True,
            timeout_seconds=args.get("timeout_seconds", consts.STATUS_TIMEOUT),
        )
        print(reformat.hexify(deploy_info))



# -*- coding: utf-8 -*-
from typing import Dict

from casperlabs_client import CasperLabsClient, io
from casperlabs_client.decorators import guarded_command

NAME: str = "show-block"
HELP: str = (
    "View properties of a block known by Casper on an existing running node. "
    "Output includes: parent hashes, storage contents of the tuplespace."
)
OPTIONS = [[("hash",), dict(type=str, help="the hash value of the block")]]


@guarded_command
def method(casperlabs_client: CasperLabsClient, args: Dict):
    response = casperlabs_client.show_block(args.get("hash"), full_view=True)
    return io.print_block(response)



# -*- coding: utf-8 -*-
import logging

import grpc

from casperlabs_client.crypto import extract_common_name
from casperlabs_client.io import read_binary_file
from casperlabs_client.decorators import retry_unary, retry_stream


class SecureGRPCService:
    def __init__(self, host, port, serviceStub, node_id, certificate_file):
        self.address = f"{host}:{port}"
        self.serviceStub = serviceStub
        self.node_id = node_id or extract_common_name(certificate_file)
        self.certificate_file = certificate_file
        file_contents = read_binary_file(self.certificate_file)
        self.credentials = grpc.ssl_channel_credentials(file_contents)
        self.secure_channel_options = (
            (
                ("grpc.ssl_target_name_override", self.node_id),
                ("grpc.default_authority", self.node_id),
            )
            if self.node_id
            else None
        )

    def __getattr__(self, name):
        logging.debug(
            f"Creating secure connection to {self.address} ({self.serviceStub})"
        )

        @retry_unary
        def unary_unary(*args):
            with grpc.secure_channel(
                self.address, self.credentials, options=self.secure_channel_options
            ) as channel:
                return getattr(self.serviceStub(channel), name)(*args)

        @retry_stream
        def unary_stream(*args):
            with grpc.secure_channel(
                self.address, self.credentials, options=self.secure_channel_options
            ) as channel:
                yield from getattr(self.serviceStub(channel), name[: -len("_stream")])(
                    *args
                )

        if name.endswith("_stream"):
            return unary_stream
        else:
            return unary_unary



# -*- coding: utf-8 -*-
from typing import Dict

from casperlabs_client.decorators import guarded_command
from casperlabs_client import io, CasperLabsClient

NAME: str = "show-blocks"
HELP: str = "View list of blocks in the current Casper view on an existing running node."
OPTIONS = [
    [
        ("-d", "--depth"),
        dict(required=True, type=int, help="depth in terms of block height"),
    ]
]


@guarded_command
def method(casperlabs_client: CasperLabsClient, args: Dict):
    response = casperlabs_client.show_blocks(args.get("depth"), full_view=False)
    io.print_blocks(response)



# -*- coding: utf-8 -*-
from typing import Dict

from casperlabs_client import CasperLabsClient, reformat
from casperlabs_client.decorators import guarded_command

NAME: str = "show-peers"
HELP: str = "Show peers connected to the node."
OPTIONS = []


@guarded_command
def method(casperlabs_client: CasperLabsClient, _: Dict):
    peers = casperlabs_client.show_peers()
    i = 0
    for i, node in enumerate(peers, 1):
        print(f"------------- node {i} ---------------")
        print(reformat.hexify(node))
    print("-----------------------------------------------------")
    print(f"count: {i}")



# -*- coding: utf-8 -*-
DEFAULT_HOST = "localhost"
DEFAULT_PORT = 40401
DEFAULT_INTERNAL_PORT = 40402
STATUS_CHECK_DELAY = 2
STATUS_TIMEOUT = 180  # 3 minutes
VISUALIZE_DAG_STREAM_DELAY = 5

ED25519_KEY_ALGORITHM = "ed25519"
SECP256K1_KEY_ALGORITHM = "secp256k1"
# To be used in ECO-463 with SECR256K1_SECURE_ENCLAVE_KEY_ALGORITHM = "secr256k1"
SUPPORTED_KEY_ALGORITHMS = (ED25519_KEY_ALGORITHM, SECP256K1_KEY_ALGORITHM)

DEFAULT_KEY_FILENAME_PREFIX: str = "account"
PRIVATE_KEY_FILENAME_SUFFIX: str = "-private.pem"
PUBLIC_KEY_FILENAME_SUFFIX: str = "-public.pem"
ACCOUNT_HASH_HEX_FILENAME_SUFFIX: str = "-id-hex"
ACCOUNT_HASH_BASE64_FILENAME_SUFFIX: str = "-id"
ACCOUNT_PRIVATE_KEY_HEX_FILENAME_SUFFIX: str = "-pk-hex"
ACCOUNT_PRIVATE_KEY_BASE64_FILENAME_SUFFIX: str = "-pk"

ACCOUNT_HASH_LENGTH: int = 32

VALIDATOR_FILENAME_PREFIX: str = "validator"
NODE_PRIVATE_KEY_FILENAME = "node.key.pem"
NODE_CERTIFICATE_FILENAME = "node.certificate.pem"
NODE_ID_FILENAME = "node-id"
